# -*- coding: utf-8 -*-

"""
####################################################################
#                                                                  #
#                           AWI Basemap                            #
#                          CONFIGURATION                           #
#                                                                  #
####################################################################

This script holds the configuration for the AWI Basemap creation
process. Adjust the settings below and the other basemap python
scripts will source these settings. The following can be adjusted
here:

- Input folder paths
- Input file paths
- General create options (like resolution, extent, etc.)
- Paths to temporary files
- Output file paths
- Operational parameters for the commands
- A small set of functions used by the other scripts

"""


#=============================================================================
#    SCRIPT INFO
#=============================================================================

__author__ = 'Simon Dreutter'
__version__ = '0.1'
__date__ = '2020-12-17'
__email__ = 'simon.dreutter@awi.de'
__status__ = 'Developement'


#=============================================================================
#    IMPORT
#=============================================================================

# import standard libraries
import os


#=============================================================================
#    INPUT FOLDERS
#=============================================================================

# These folders are part of the processing infrastructure. Only change the
# variables if the directory names or locations have been changed.

# path to data directory
PATH_DATA = os.path.abspath(os.path.join('..','data'))

# path to separate datasets
PATH_ADD = os.path.join(PATH_DATA,'add')
PATH_GEBCO = os.path.join(PATH_DATA,'gebco')
PATH_GIMP = os.path.join(PATH_DATA,'gimp')
PATH_GLIMS = os.path.join(PATH_DATA,'glims')

# path to result directory
PATH_RESULT = os.path.abspath(os.path.join('..','result'))


#=============================================================================
#    INPUT FILES
#=============================================================================

# These file paths are using the current version of the input files' names. If
# any of the input files are updated in the data directory, their filenames
# need to updated in this section.

# input filepath to ADD Coastlines
ADD_COASTLINE_IN = os.path.join(PATH_ADD,'add_coastline_medium_res_polygon_v7.3.gpkg')
# input filepath to ADD Rock Outcrops
ADD_ROCK_OUTCROP_IN = os.path.join(PATH_ADD,'add_rock_outcrop_medium_res_polygon_v7.3.gpkg')

# input filepath to GEBCO netCDF grid
GEBCO_IN = os.path.join(PATH_GEBCO,'GEBCO_2020.nc')

# input filepath to GIMP GeoTIFF
GIMP_IN = os.path.join(PATH_GIMP,'GimpIceMask_90m_v1.1.tif')

# input filepath to GLIMS Esri Shapefile
GLIMS_IN = os.path.join(PATH_GLIMS,'glims_polygons.shp')

# color palettes
CP_DEFAULT = os.path.abspath(os.path.join('..','styles','color_palettes','basemap_default.txt'))
CP_BRIGHT = os.path.abspath(os.path.join('..','styles','color_palettes','basemap_bright.txt'))
CP_DARK = os.path.abspath(os.path.join('..','styles','color_palettes','basemap_dark.txt'))


#=============================================================================
#    GENERAL CREATE OPTIONS
#=============================================================================

# Here are some general options for the resulting grids like resolution and
# shading exaggeration. Adjusting the resolution of the output grid will
# directly affect file sizes. The maximum resolution for the World map is the
# original GEBCO resolution (15 arc seconds / 0.00416... degree). For the
# Polar maps, higher resolutions than 500m can be achieved theoretically, but
# this will result in some ugly artifacts (from warping) in the lower
# latitudes.
#
# The best tradeoff between file size, resolution and artifacts is
#   World: 30 arc seconds / 0.0083... degree
#   Polar: 1000 meter
#
# The extent of the Polar maps should be kept to an acceptable minium (larger
# number) to reduce file size and atrifacts from warping in the lower
# latitudes.

# Final AWI Basemap (World) resolution
# 15 arc seconds (original GEBCO resolution)
#RESOLUTION_DEG = 0.004166666666666666
# 30 arc seconds
RESOLUTION_DEG = 0.008333333333333333

# Final AWI Basemap (Arctic and Antarctic) resolution
# pendant to 15 arc seconds
#RESOLUTION_M = 500.0
# pendant to 30 arc seconds
RESOLUTION_M = 1000.0

# GIMP Resolution
# half the GEBCO resolution
GIMP_RESOLUTION = RESOLUTION_DEG / 2

# Hillshade z factor / vertical exaggeration
Z_FACTOR = 10.0

# Arctic extent down to X degree latitude (positive)
ARCTIC_EXTENT_LAT = 30.0

# Antarctic extent up to X degree latitude (negative)
ANTARCTIC_EXTENT_LAT= -30.0


#=============================================================================
#    TEMPORARY FILES
#=============================================================================

# These are file paths to temporary files that will be created (and cleaned
# up) during the process. The varibales care used throughout the different
# scripts.

# ADD:
# ADD land with cut rock outcrops
ADD_ICE_SHEET = os.path.join(PATH_ADD,'add_ice_sheet.gpkg')
# ADD ice mask
ADD_SHELF_ICE = os.path.join(PATH_ADD,'add_shelf_ice.gpkg')
# ADD rock outcrops
ADD_ROCK_OUTCROP = os.path.join(PATH_ADD,'add_rock_outcrop.gpkg')

#=============================================================================

# GIMP:
# GIMP raster
GIMP_RASTER = os.path.join(PATH_GIMP,'GimpIceMask.tif')
# GIMP vector
GIMP = os.path.join(PATH_GIMP,'GimpIceMask.gpkg')

#=============================================================================

# GLIMS:
# GLIMS World
GLIMS_WORLD = os.path.join(PATH_GLIMS,'glims_world.gpkg')
# GLIMS Arctic
GLIMS_ARCTIC = os.path.join(PATH_GLIMS,'glims_arctic.gpkg')
# GLIMS Antarctic
GLIMS_ANTARCTIC = os.path.join(PATH_GLIMS,'glims_antarctic.gpkg')

#=============================================================================

# GEBCO:
# GEBCO World
GEBCO_WORLD = os.path.join(PATH_GEBCO,'GEBCO_world.tif')
GEBCO_WORLD_HILLSHADE = os.path.join(PATH_GEBCO,'GEBCO_world_hillshade.tif')

# GEBCO Arctic
GEBCO_ARCTIC_CLIP_VRT = os.path.join(PATH_GEBCO,'GEBCO_arctic_clip.vrt')
GEBCO_ARCTIC_FULL_VRT = os.path.join(PATH_GEBCO,'GEBCO_arctic_full.vrt')
GEBCO_ARCTIC = os.path.join(PATH_GEBCO,'GEBCO_arctic.tif')
GEBCO_ARCTIC_HILLSHADE = os.path.join(PATH_GEBCO,'GEBCO_arctic_hillshade.tif')

# GEBCO Antarctic
GEBCO_ANTARCTIC_CLIP_VRT = os.path.join(PATH_GEBCO,'GEBCO_antarctic_clip.vrt')
GEBCO_ANTARCTIC_FULL_VRT = os.path.join(PATH_GEBCO,'GEBCO_antarctic_full.vrt')
GEBCO_ANTARCTIC = os.path.join(PATH_GEBCO,'GEBCO_antarctic.tif')
GEBCO_ANTARCTIC_HILLSHADE = os.path.join(PATH_GEBCO,'GEBCO_antarctic_hillshade.tif')

# temporary files
TMP_RGB = os.path.join(PATH_GEBCO,'tmp_rgb.tif')
TMP_SHADED = os.path.join(PATH_GEBCO,'tmp_shaded.tif')

#=============================================================================

# Final Output:
# default colors
BASEMAP_WORLD = os.path.join(PATH_RESULT,'AWI_Basemap_World.tif')
BASEMAP_ARCTIC = os.path.join(PATH_RESULT,'AWI_Basemap_Arctic.tif')
BASEMAP_ANTARCTIC = os.path.join(PATH_RESULT,'AWI_Basemap_Antarctic.tif')

# bright colors
BASEMAP_WORLD_BRIGHT = os.path.join(PATH_RESULT,'AWI_Basemap_World_bright.tif')
BASEMAP_ARCTIC_BRIGHT = os.path.join(PATH_RESULT,'AWI_Basemap_Arctic_bright.tif')
BASEMAP_ANTARCTIC_BRIGHT = os.path.join(PATH_RESULT,'AWI_Basemap_Antarctic_bright.tif')

# dark colors
BASEMAP_WORLD_DARK = os.path.join(PATH_RESULT,'AWI_Basemap_World_dark.tif')
BASEMAP_ARCTIC_DARK = os.path.join(PATH_RESULT,'AWI_Basemap_Arctic_dark.tif')
BASEMAP_ANTARCTIC_DARK = os.path.join(PATH_RESULT,'AWI_Basemap_Antarctic_dark.tif')


#=============================================================================
#    PROJECTIONS
#=============================================================================

# The EPSG codes for the three map projections only need to be changed, if a
# different projection is required for the Map Viewers.

# EPSG code for World Basemap
EPSG_WORLD = 'EPSG:4326' # WGS84 / Geographic

# EPSG code for Arctic Basemap
EPSG_ARCTIC = 'EPSG:3995' # WGS 84 / Arctic Polar Stereographic

# EPSG code for Antarctic Basemap
EPSG_ANTARCTIC = 'EPSG:3031' # WGS 84 / Antarctic Polar Stereographic


#=============================================================================
#    OPERATIONAL PARAMETERS
#=============================================================================

# The following settings are operational parameters for the process. If, e.g.
# your Python (v3.x) installation has a different executable, like "python3"
# as it comes with the OSGeo4W Python installation, some scripts need to know
# that and the command needs to be changed here.
# And if the temporary GeoTIFFs created by GDAL shall be created with
# different create options, they can be adjusted below. The create options for
# the final results, however, are specified in the last python script.

# Python executable
PYTHON = 'python3'

# GDAL creation options
GDAL_CREATE_OPTIONS = '-co COMPRESS=NONE -co BIGTIFF=IF_NEEDED'
#GDAL_CREATE_OPTIONS = '-co COMPRESS=DEFLATE -co PREDICTOR=2 -co ZLEVEL=9 -co BIGTIFF=YES'

# GDAL creation options for python modules
GDAL_CREATE_OPTIONS_PY = '--co COMPRESS=NONE --co BIGTIFF=IF_NEEDED'
#GDAL_CREATE_OPTIONS_PY = '--co COMPRESS=DEFLATE --co PREDICTOR=2 --co ZLEVEL=9 --co BIGTIFF=YES'

# GDAL warp options
WARP_OPTIONS = '-wo SOURCE_EXTRA=1000 -et 0'

#=============================================================================
#    RUN FUNCTION
#=============================================================================

# function to run a shell command and print a message to stdout
def run(msg,cmd):
    print('============================================================')
    print()
    print(msg)
    print()
    print(cmd)
    print()
    os.system(cmd)
    print()


#=============================================================================
#    CLEANUP
#=============================================================================

# list of temporary files that shall not be deleted
DONT_DELETE = [
#    GEBCO_WORLD,
#    GEBCO_WORLD_HILLSHADE,
#    GEBCO_ARCTIC,
#    GEBCO_ARCTIC_HILLSHADE,
#    GEBCO_ANTARCTIC,
#    GEBCO_ANTARCTIC_HILLSHADE
]

# function to cleanup temporary files if they are not in the DONT_DELETE list
def cleanup(dataset):
    if dataset not in DONT_DELETE:
        if os.path.isfile(dataset):
            os.remove(dataset)
